package circularlist;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class CircularListImpl<E> implements CircularList<E> {
	private List<E> elements;
	private int index = -1;

	@SafeVarargs
	public CircularListImpl(E... elements) {
		this.elements = new ArrayList<E>(Arrays.asList(elements));
	}

	/* (non-Javadoc)
	 * @see circularlist.CircularList#get(int)
	 */
	@Override
	public E get(int i) {
		return elements.get(i % elements.size());
	}

	/* (non-Javadoc)
	 * @see circularlist.CircularList#indexOf(java.lang.Object)
	 */
	@Override
	public int indexOf(Object o) throws ObjectMissingException {
		int i = elements.indexOf(o);
		if (i == -1)
			throw new ObjectMissingException(o);
		return i;
	}

	/* (non-Javadoc)
	 * @see circularlist.CircularList#size()
	 */
	@Override
	public int size() {
		return elements.size();
	}

	/* (non-Javadoc)
	 * @see circularlist.CircularList#nextElement()
	 */
	@Override
	public E nextElement() {
		index = ++index % elements.size();
		return elements.get(index);
	}

	/* (non-Javadoc)
	 * @see circularlist.CircularList#add(E)
	 */
	@Override
	public boolean add(E element) {
		return elements.add(element);
	}
	
	/* (non-Javadoc)
	 * @see circularlist.CircularList#add(int, E)
	 */
	@Override
	public void add(int i, E element) {
		elements.add(i, element);
	}
	
	/* (non-Javadoc)
	 * @see circularlist.CircularList#addAll(circularlist.CircularListImpl)
	 */
	@Override
	public void addAll(CircularListImpl<? extends E> cl) {
		elements.addAll(cl.elements);
	}
	
	public String toString() {
		return "Circular list of size " + size();
	}

	public boolean equals(Object o) {
		if (!(o instanceof CircularListImpl))
			return false;
		CircularListImpl<?> cl = (CircularListImpl<?>) o;
		return elements.equals(cl.elements);
	}

	public int hashCode() {
		return elements.hashCode();
	}
}